//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/SamplesSet.cpp
//! @brief     Implements class SamplesSet.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/SamplesSet.h"
#include "GUI/Model/Sample/ItemWithMaterial.h"
#include "GUI/Model/Sample/SampleItem.h"
#include "GUI/Support/XML/Backup.h"
#include "GUI/Support/XML/UtilXML.h"
#include <QApplication>

namespace {
namespace Tag {

const QString Sample("Sample");
const QString CurrentIndex("CurrentIndex");

} // namespace Tag
} // namespace

QStringList SamplesSet::sampleNames() const
{
    QStringList existingNames;
    for (const auto* item : *this)
        existingNames << item->sampleName();
    return existingNames;
}

void SamplesSet::writeTo(QXmlStreamWriter* w) const
{
    XML::writeAttribute(w, XML::Attrib::version, uint(1));

    // samples
    for (const auto* sample : *this) {
        w->writeStartElement(Tag::Sample);
        sample->writeTo(w);
        w->writeEndElement();
    }

    // selected index
    w->writeStartElement(Tag::CurrentIndex);
    XML::writeAttribute(w, XML::Attrib::value, currentIndex());
    w->writeEndElement();
}

void SamplesSet::readFrom(QXmlStreamReader* r)
{
    clear();

    const uint version = XML::readUIntAttribute(r, XML::Attrib::version);
    Q_UNUSED(version)

    while (r->readNextStartElement()) {
        QString tag = r->name().toString();

        // sample
        if (tag == Tag::Sample) {
            auto* t = new SampleItem;
            t->readFrom(r);
            push_back(t);
            XML::gotoEndElementOfTag(r, tag);

            // selected index
        } else if (tag == Tag::CurrentIndex) {
            size_t i;
            XML::readAttribute(r, XML::Attrib::value, &i);
            setCurrentIndex(i);
            XML::gotoEndElementOfTag(r, tag);

        } else
            r->skipCurrentElement();
    }
}
