//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Data/DatafilesQModel.h
//! @brief     Defines class DatafilesQModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_DATA_DATAFILESQMODEL_H
#define BORNAGAIN_GUI_VIEW_DATA_DATAFILESQMODEL_H

#include <QAbstractListModel>

class DatafileItem;

//! Tree representation of DatafilesSet, for use in DatafilesSelector.

class DatafilesQModel : public QAbstractListModel {
    Q_OBJECT
public:
    explicit DatafilesQModel(QObject* parent);

    void injectDatafileItem(DatafileItem* item);
    void removeDatafileItemAt(int row);

    int rowCount(const QModelIndex& parent = {}) const override;
    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;

    DatafileItem* itemForIndex(const QModelIndex& index);
    QModelIndex indexForItem(const DatafileItem* item) const;
    const DatafileItem* topMostItem() const; //!< The topmost visible item. Can be null.
    bool isHeadline(const QModelIndex&) const { return false; }
};

#endif // BORNAGAIN_GUI_VIEW_DATA_DATAFILESQMODEL_H
